#! /usr/local/bin/python3
'''
    A python script to help run tracking benchmarks
    simulation -> reconstruction -> analysis
    Author: Chao Peng (ANL)
    Date: 07/20/2021
'''
import os
import sys
import subprocess
import argparse


default_compact = os.path.join(
        os.environ.get('JUGGLER_DETECTOR_PATH',  os.environ.get('DETECTOR_PATH', '')),
        '{}.xml'.format(os.environ.get('JUGGLER_DETECTOR', '')))

parser = argparse.ArgumentParser()
parser.add_argument('-n', '--numberOfEvents', dest='nev', type=int, default=100, help='Number of events to process.')
parser.add_argument('-t', '--nametag', type=str, default='TRACKING', help='Name tag for output files.')
parser.add_argument('--seed', type=int, default=-1, help='Random seed to scripts.')
parser.add_argument('--process', type=str, default='sim, rec, ana', help='Processes to be executed (sim, rec, ana).')
parser.add_argument('--numberOfLayers', dest='nlayer', type=int, default=20, help='number of layers in ML data.')
parser.add_argument('--numberOfHits', dest='nhit', type=int, default=20, help='number of hits in ML data.')
parser.add_argument('--particles', type=str, default='electron', help='Partcile names, separated by \",\".')
parser.add_argument('--etamin', type=float, default=-4, help='Minimum pseudorapidity.')
parser.add_argument('--etamax', type=float, default=4, help='Maximum pseudorapidity.')
parser.add_argument('--pmin', type=float, default=5.0, help='Minimum momentum of particles (GeV).')
parser.add_argument('--pmax', type=float, default=5.0, help='Maximum momentum of particles (GeV).')
parser.add_argument('--compact', type=str, default=default_compact, help='Path to detector compact file.')
parser.add_argument('--options', type=str, default='truth_seeded_tracking.py', help='Path to reconstruction options script.')
parser.add_argument('--uploadSizeLimit', type=float, default=10, help='Upper limit of file size (Mb) to be uploaded.')

args = parser.parse_args()
kwargs = vars(args)

script_dir = os.path.dirname(os.path.realpath(__file__))
gen_script = os.path.join(script_dir, 'scripts', 'gen_particles.py')
option_script = os.path.join(script_dir, 'options', args.options)
analysis_script = os.path.join(script_dir, 'scripts', 'tracking_performance.py')

gen_file = 'gen_{nametag}_{pmin}_{pmax}.hepmc'.format(**kwargs)
sim_file = 'sim_{nametag}_{pmin}_{pmax}.root'.format(**kwargs)
rec_file = 'rec_{nametag}_{pmin}_{pmax}.root'.format(**kwargs)

procs = [p.strip() for p in args.process.split(',')]
sdir = os.path.dirname(os.path.realpath(__file__))

if 'sim' in procs:
    # generate particles
    gen_cmd = ['python', gen_script, gen_file,
            '-n', '{}'.format(args.nev),
            '-s', '{}'.format(args.seed),
            '--etamin', '{}'.format(args.etamin), '--etamax', '{}'.format(args.etamax),
            '--pmin', '{}'.format(args.pmin), '--pmax', '{}'.format(args.pmax),
            '--particles', args.particles]
    subprocess.run(gen_cmd)
    # simulation
    sim_cmd = ['npsim',
            '--part.minimalKineticEnergy', '1*TeV',
            '--numberOfEvents', '{}'.format(args.nev),
            '--runType', 'batch',
            '--inputFiles', gen_file,
            '--outputFile', sim_file,
            '--compact', args.compact,
            '-v', 'WARNING']
    if args.seed > 0:
        sim_cmd += ['--random.seed', args.seed]
    return_code = subprocess.run(sim_cmd).returncode
    if return_code is not None and return_code < 0:
        print("ERROR running simulation!")
        exit(1)
    subprocess.run(['rootls', '-t', sim_file])


if 'rec' in procs:
    # export to environment variables (used to pass arguments to the option file)
    os.environ['JUGGLER_SIM_FILE'] = sim_file
    os.environ['JUGGLER_REC_FILE'] = rec_file
    os.environ['JUGGLER_COMPACT_PATH'] = args.compact
    os.environ['JUGGLER_N_EVENTS'] = '{}'.format(args.nev)

    juggler_xenv = os.path.join(os.environ.get('JUGGLER_INSTALL_PREFIX', '../local'), 'Juggler.xenv')

    rec_cmd = ['xenv', '-x', juggler_xenv, 'gaudirun.py', os.path.join(sdir, 'options', option_script)]
    return_code = subprocess.run(rec_cmd).returncode
    if return_code is not None and return_code < 0:
        print('ERROR running juggler ({})!'.format(opt))
        exit(1)
    process = subprocess.run(['rootls', '-t', rec_file])


if 'ana' in procs:
    os.makedirs('results', exist_ok=True)
    ana_cmd = ['python', analysis_script, rec_file,
               '--mc-collection', 'mcparticles2',
               '--tracking-collection', 'outputTrackParameters',
               '-o', 'results', '-t', args.nametag]
    return_code = subprocess.run(ana_cmd).returncode
    if return_code is not None and return_code < 0:
        print('ERROR running analysis ({})!'.format(ana))
        exit(1)


# upload generated data files if it was small enough (< 10 Mb)
for upload in [rec_file]:
    process = subprocess.run(['stat', '--format=%s', upload], stdout=subprocess.PIPE)
    size = int(process.stdout)/1024./1024.
    if size > args.uploadSizeLimit:
        print('Skip uploading file \"{}\" because its size ({:.2f} Mb) > {:.2f} Mb'\
              .format(upload, size, args.uploadSizeLimit))
    else:
        os.system('cp {} results/.'.format(upload))
        print('Upload file \"{}\", size = {:.2f} Mb'.format(upload, size))

